import json
import pathlib

from data_inclusion.schema import (
    Frais,
    LabelNational,
    ModeAccueil,
    ModeOrientationAccompagnateur,
    ModeOrientationBeneficiaire,
    Profil,
    Service,
    Structure,
    Thematique,
    Typologie,
    TypologieService,
    ZoneDiffusionType,
)


def main():
    output_dir = pathlib.Path() / "schemas"
    output_dir.mkdir(exist_ok=True)

    with (output_dir / "structures.json").open("w") as file:
        json.dump(
            Structure.model_list_json_schema(
                title="Structures de l'insertion",
                id=(
                    "https://raw.githubusercontent.com/betagouv/data-inclusion-schema"
                    "/main/structures.json"
                ),
                description="",
            ),
            file,
            indent=2,
        )

    with (output_dir / "services.json").open("w") as file:
        json.dump(
            Service.model_list_json_schema(
                title="Services de l'insertion",
                id=(
                    "https://raw.githubusercontent.com/betagouv/data-inclusion-schema"
                    "/main/services.json"
                ),
                description="",
            ),
            file,
            indent=2,
        )

    # fichiers supplémentaires documentant les énumérations

    (output_dir / "extra").mkdir(exist_ok=True)

    enum_x_file_tuples_list = [
        ("frais.json", Frais),
        ("labels-nationaux.json", LabelNational),
        ("modes-accueil.json", ModeAccueil),
        ("modes-orientation-accompagnateur.json", ModeOrientationAccompagnateur),
        ("modes-orientation-beneficiaire.json", ModeOrientationBeneficiaire),
        ("profils.json", Profil),
        ("thematiques.json", Thematique),
        ("typologies-de-services.json", TypologieService),
        ("typologies-de-structures.json", Typologie),
        ("zones-de-diffusion-types.json", ZoneDiffusionType),
    ]

    for filename, enum in enum_x_file_tuples_list:
        with (output_dir / "extra" / filename).open("w") as file:
            json.dump(
                enum.as_dict_list(),
                file,
                indent=2,
                ensure_ascii=False,
            )


if __name__ == "__main__":
    """
    Usage:
        $ python -m data_inclusion.schema
    """
    main()



from data_inclusion.schema.base import EnhancedEnum


class ModeOrientationAccompagnateur(EnhancedEnum):
    COMPLETER_LE_FORMULAIRE_DADHESION = (
        "completer-le-formulaire-dadhesion",
        "Compléter le formulaire d’adhésion",
        None,
    )
    ENVOYER_UN_MAIL = (
        "envoyer-un-mail",
        "Envoyer un mail",
        None,
    )
    ENVOYER_UN_MAIL_AVEC_UNE_FICHE_DE_PRESCRIPTION = (
        "envoyer-un-mail-avec-une-fiche-de-prescription",
        "Envoyer un mail avec une fiche de prescription",
        None,
    )
    ENVOYER_UN_MAIL_AVEC_DES_DOCUMENTS_A_COMPLETER = (
        "envoyer-un-mail-avec-des-documents-a-completer",
        "Envoyer un mail avec des documents à compléter",
        None,
    )
    TELEPHONER = (
        "telephoner",
        "Téléphoner",
        None,
    )
    AUTRE = (
        "autre",
        "Autre",
        None,
    )


class ModeOrientationBeneficiaire(EnhancedEnum):
    COMPLETER_LE_FORMULAIRE_DADHESION = (
        "completer-le-formulaire-dadhesion",
        "Compléter le formulaire d’adhésion",
        None,
    )
    ENVOYER_UN_MAIL = (
        "envoyer-un-mail",
        "Envoyer un mail",
        None,
    )
    SE_PRESENTER = (
        "se-presenter",
        "Se présenter",
        None,
    )
    TELEPHONER = (
        "telephoner",
        "Téléphoner",
        None,
    )
    AUTRE = (
        "autre",
        "Autre",
        None,
    )



from data_inclusion.schema.base import EnhancedEnum


class ModeAccueil(EnhancedEnum):
    A_DISTANCE = (
        "a-distance",
        "À distance",
        None,
    )
    EN_PRESENTIEL = (
        "en-presentiel",
        "En présentiel",
        None,
    )



from data_inclusion.schema.base import EnhancedEnum


class Profil(EnhancedEnum):
    SENIORS_65 = (
        "seniors-65",
        "Seniors (+ 65 ans)",
        None,
    )
    FAMILLES_ENFANTS = (
        "familles-enfants",
        "Familles/enfants",
        None,
    )
    ADULTES = (
        "adultes",
        "Adultes",
        None,
    )
    JEUNES_16_26 = (
        "jeunes-16-26",
        "Jeunes (16-26 ans)",
        None,
    )
    PUBLIC_LANGUES_ETRANGERES = (
        "public-langues-etrangeres",
        "Public langues étrangères",
        None,
    )
    DEFICIENCE_VISUELLE = (
        "deficience-visuelle",
        "Déficience visuelle",
        None,
    )
    SURDITE = (
        "surdite",
        "Surdité",
        None,
    )
    HANDICAPS_PSYCHIQUES = (
        "handicaps-psychiques",
        (
            "Handicaps psychiques : troubles psychiatriques donnant lieu à des"
            " atteintes comportementales"
        ),
        None,
    )
    HANDICAPS_MENTAUX = (
        "handicaps-mentaux",
        "Handicaps mentaux : déficiences limitant les activités d’une personne",
        None,
    )
    FEMMES = (
        "femmes",
        "Femmes",
        "Le lieu propose des accompagnements réservés aux femmes.",
    )
    PERSONNES_HANDICAPEES = (
        "personnes-handicapees",
        "Personnes handicapées",
        (
            "Ce terme est privilégié pour désigner une incapacité persistante et "
            "significative subie par une personne dans son environnement."
        ),
    )
    PERSONNES_EN_SITUATION_DE_HANDICAP = (
        "personnes-en-situation-de-handicap",
        "Personnes en situation de handicap",
        (
            "Une personne se déplaçant temporairement en béquilles des suites d’une "
            "intervention médicale est en situation de handicap sans toutefois être "
            "handicapée."
        ),
    )
    PERSONNES_EN_SITUATION_ILLETTRISME = (
        "personnes-en-situation-illettrisme",
        "Personnes en situation d’illettrisme",
        None,
    )
    BENEFICIAIRES_RSA = (
        "beneficiaires-rsa",
        "Bénéficiaires du Revenu de Solidarité Active (RSA)",
        None,
    )
    DEMANDEURS_EMPLOI = (
        "demandeurs-demploi",
        "Demandeurs ou demandeuses d’emploi",
        None,
    )



from datetime import date, datetime
from typing import Optional

from pydantic import EmailStr, HttpUrl, StringConstraints
from typing_extensions import Annotated

from data_inclusion.schema import common
from data_inclusion.schema.base import BaseModel
from data_inclusion.schema.frais import Frais
from data_inclusion.schema.modes_accueil import ModeAccueil
from data_inclusion.schema.modes_orientation import (
    ModeOrientationAccompagnateur,
    ModeOrientationBeneficiaire,
)
from data_inclusion.schema.profils import Profil
from data_inclusion.schema.thematiques import Thematique
from data_inclusion.schema.typologies_de_services import TypologieService
from data_inclusion.schema.zones_de_diffusion import ZoneDiffusionType


class Service(BaseModel):
    # fields
    id: str
    structure_id: str
    source: str
    nom: str
    presentation_resume: Optional[
        Annotated[str, StringConstraints(max_length=280)]
    ] = None
    presentation_detail: Optional[str] = None
    types: Optional[list[TypologieService]] = None
    thematiques: Optional[list[Thematique]] = None
    prise_rdv: Optional[HttpUrl] = None
    frais: Optional[list[Frais]] = None
    frais_autres: Optional[str] = None
    profils: Optional[list[Profil]] = None
    pre_requis: Optional[list[str]] = None
    cumulable: Optional[bool] = None
    justificatifs: Optional[list[str]] = None
    formulaire_en_ligne: Optional[HttpUrl] = None
    commune: Optional[str] = None
    code_postal: Optional[common.CodePostal] = None
    code_insee: Optional[common.CodeCommune] = None
    adresse: Optional[str] = None
    complement_adresse: Optional[str] = None
    longitude: Optional[float] = None
    latitude: Optional[float] = None
    recurrence: Optional[str] = None
    date_creation: Optional[date] = None
    date_suspension: Optional[date] = None
    lien_source: Optional[HttpUrl] = None
    telephone: Optional[str] = None
    courriel: Optional[EmailStr] = None
    contact_public: Optional[bool] = None
    date_maj: Optional[date | datetime] = None
    modes_accueil: Optional[list[ModeAccueil]] = None
    zone_diffusion_type: Optional[ZoneDiffusionType] = None
    zone_diffusion_code: Optional[
        common.CodeCommune
        | common.CodeEPCI
        | common.CodeDepartement
        | common.CodeRegion
    ] = None
    zone_diffusion_nom: Optional[str] = None
    contact_nom_prenom: Optional[str] = None
    modes_orientation_beneficiaire: Optional[list[ModeOrientationBeneficiaire]] = None
    modes_orientation_beneficiaire_autres: Optional[str] = None
    modes_orientation_accompagnateur: Optional[
        list[ModeOrientationAccompagnateur]
    ] = None
    modes_orientation_accompagnateur_autres: Optional[str] = None
